@@ -26,6 +26,7 @@
 
 #include "dunbash.h"
 #include "combat.h"
+#include "monsters.h"
 
 int player_attack(int dy, int dx)
 {
@@ -75,7 +76,7 @@ int uhitm(int mon)
 	{
 		damage = u.body / 10;
 	}
-	if (u.ring)
+	if (u.ring != -1)
 	{
 		ring = objects + u.ring;
 		pring = permobjs + ring->obj_id;
@@ -125,6 +126,10 @@ int uhitm(int mon)
 	}
 	print_msg("You do %d damage.\n", damage);
 	damage_mon(mon, damage, 1);
+	if (u.weapon != -1)
+	{
+		damage_obj(u.weapon);
+	}
 	return 1;	/* Hit. */
 }
 
@@ -163,6 +168,7 @@ int ushootm(int sy, int sx)
 				print_msg("You hit ");
 				print_mon_name(mapmonster[y][x], 1);
 				print_msg(".\n");
+				print_msg("You do %d damage.\n", damage);
 				damage_mon(mapmonster[y][x], damage, 1);
 				return 1;
 			}
@@ -189,16 +195,34 @@ int mhitu(int mon, enum damtyp dtype)
 	int damage;
 	int unaffected;
 	struct mon *mptr = monsters + mon;
-	print_mon_name(mon, 3);
 	tohit = zero_die(mptr->mtohit + 5);
 	if (tohit < u.defence)
 	{
-		print_msg(" misses you.\n");
+		/* Note: Yes, all attacks can damage your armour. Deal. */
+		if ((u.armour != -1) && (tohit > agility_modifier()))
+		{
+			/* Monster hit your armour. */
+			print_msg("Your armour deflects the blow.\n");
+			damage_obj(u.armour);
+		}
+		else
+		{
+			print_mon_name(mon, 3);
+			print_msg(" misses you.\n");
+		}
 		return 0;
 	}
 	damage = one_die(mptr->mdam);
 	unaffected = player_resists_dtype(dtype);
+	print_mon_name(mon, 3);
 	print_msg(" hits you.\n");
+	if (u.armourmelt && (!zero_die(3)))
+	{
+		/* If you're subject to armourmelt, it is decreed that one
+		 * blow in three hits your dust-weak armour and rips a chunk
+		 * out of it. */
+		damage_obj(u.armour);
+	}
 test_unaffected:
 	if (unaffected)
 	{
@@ -260,11 +284,10 @@ test_unaffected:
 		print_msg("You take %d damage.\n", damage);
 		if ((mptr->mon_id == PM_VAMPIRE) && !player_resists_dtype(DT_NECRO))
 		{
-			heal_mon(mon, damage * 2 / 5);
-		}
-		if ((tohit - u.defence >= 10) && (mptr->mon_id == PM_SNAKE))
+			heal_mon(mon, damage * 2 / 5, 1);
+		} else if ((tohit - u.defence >= 5) && (mptr->mon_id == PM_SNAKE))
 		{
-			drain_body(1, "snake venom");
+			drain_body(1, "snake venom", 0);
 		}
 		damage_u(damage, DEATH_KILLED_MON, permons[mptr->mon_id].name);
 		display_update();
