@@ -291,6 +291,20 @@ int inv_select(enum poclass_num filter, const char *action, int accept_blank)
 {
 	int selection;
 	int ch;
+	int i;
+	int items = 0;
+	for (i = 0; i < 19; i++)
+	{
+		if ((u.inventory[i] != -1) && ((filter == POCLASS_NONE) || (permobjs[objects[u.inventory[i]].obj_id].poclass == filter)))
+		{
+			items++;
+		}
+	}
+	if (items == 0)
+	{
+		print_msg("You have nothing to %s.\n", action);
+		return -1;
+	}
 	print_msg("Items available to %s\n", action);
 	print_inv(filter);
 	if (accept_blank)
@@ -333,7 +347,7 @@ tryagain:
 	case 'r':
 	case 's':
 		selection = ch - 'a';
-		if ((filter == POCLASS_NONE) || (permobjs[objects[u.inventory[selection]].obj_id].poclass == filter))
+		if ((u.inventory[selection] != -1) && ((filter == POCLASS_NONE) || (permobjs[objects[u.inventory[selection]].obj_id].poclass == filter)))
 		{
 			return selection;
 		}
@@ -342,7 +356,6 @@ tryagain:
 		print_msg("Bad selection\n");
 		goto tryagain;
 	}
-	return selection;
 }
 
 int select_dir(int *psy, int *psx)
@@ -432,6 +445,8 @@ enum game_cmd get_command(void)
 			return GET_ITEM;
 		case 'd':
 			return DROP_ITEM;
+		case 'D':
+			return DUMP_CHARA;
 		case 'S':
 			return SAVE_GAME;
 		case 'X':
@@ -573,6 +588,7 @@ void print_help(void)
 	print_msg("S   save and exit\n");
 	print_msg("X   quit without saving\n");
 	print_msg("i   print your inventory\n");
+	print_msg("D   dump your character's details to <name>.dump\n");
 	print_msg("?   print this message\n");
 	print_msg("\nPress any key to continue...\n");
 	wgetch(message_window);
