@@ -40,7 +40,6 @@ static void rebuild_mapmons(void);
 static void main_loop(void);
 int game_finished;
 int game_tick;
-int do_command(enum game_cmd cmd);
 
 static void rebuild_mapmons(void)
 {
@@ -345,8 +344,8 @@ int do_command(enum game_cmd cmd)
 			int dmg;
 			print_msg("You remove your ring.\n");
 			print_msg("It exacts vengeance!\n");
-			drain_body(one_die(4), "a ring of doom");
-			drain_agility(one_die(4), "a ring of doom");
+			drain_body(one_die(4), "a ring of doom", 1);
+			drain_agility(one_die(4), "a ring of doom", 1);
 			dmg = one_die(20);
 			damage_u(dmg, DEATH_KILLED, "a ring of doom");
 			u.hpmax -= dmg;
@@ -413,6 +412,9 @@ int do_command(enum game_cmd cmd)
 			return 1;
 		}
 		return 0;
+	case DUMP_CHARA:
+		write_char_dump();
+		return 0;
 	case SAVE_GAME:
 		game_finished = 1;
 		save_game();
@@ -506,16 +508,31 @@ void main_loop(void)
 			/* Monsters now regenerate */
 			if (monsters[i].hpcur < monsters[i].hpmax)
 			{
+				int dy, dx, cansee;
+				dy = u.y - monsters[i].y;
+				dx = u.x - monsters[i].x;
+				if (((dy > -2) && (dy < 2) && (dx > -2) && (dx < 2)) ||
+				    ((roomnums[u.y][u.x] != -1) && (roomnums[u.y][u.x] == roomnums[monsters[i].y][monsters[i].x])))
+				{
+					cansee = 1;
+				}
+				else
+				{
+					cansee = 0;
+				}
 				if ((monsters[i].mon_id == PM_TROLL) &&
 				    !(game_tick % 10))
 				{
-					print_msg("The troll regenerates.\n");
-					heal_mon(i, one_die(3) + 3);
+					if (cansee)
+					{
+						print_msg("The troll regenerates.\n");
+					}
+					heal_mon(i, one_die(3) + 3, cansee);
 				}
 				else if ((monsters[i].mon_id != PM_TROLL) &&
 				    !(game_tick % 20))
 				{
-					heal_mon(i, 1);
+					heal_mon(i, 1, cansee);
 				}
 			}
 			if (action_speed <= permons[monsters[i].mon_id].speed)
