@@ -25,6 +25,7 @@
  */
 
 #include "dunbash.h"
+#include "monsters.h"
 
 #include <string.h>
 int mapobject[DUN_SIZE][DUN_SIZE];
@@ -33,7 +34,7 @@ enum terrain_num terrain[DUN_SIZE][DUN_SIZE];
 int mapflags[DUN_SIZE][DUN_SIZE];
 int roomnums[DUN_SIZE][DUN_SIZE];
 int depth = 1;
-static int roomlinkage[MAX_ROOMS][MAX_ROOMS];
+int roomlinkage[MAX_ROOMS][MAX_ROOMS];
 int roombounds[MAX_ROOMS][4];
 int stairs_room = -1;
 int zoo_room = -1;
