@@ -25,7 +25,6 @@
  */
 
 /* TODO: Convert missile AI to a new-style AI function. */
-/* TODO: Implement pathfinding AI for "smart" monsters. */
 #define MON2_C
 #include "dunbash.h"
 #include "bmagic.h"
@@ -41,8 +40,7 @@ struct ai_cell {
 
 /* prototypes for AI preference functions. */
 static void get_naive_prefs(int y, int x, int dy, int dx, int *pref_y, int *pref_x);
-/* static void get_seeking_prefs(int y, int x, int dy, int dx, int *pref_y, int *pref_x);
-static void get_missile_prefs(int y, int x, int dy, int dx, int *pref_y, int *pref_x); */
+ static void get_seeking_prefs(int y, int x, int dy, int dx, int *pref_y, int *pref_x);
 static void get_drunk_prefs(int y, int x, int dy, int dx, int *pref_y, int *pref_x);
 static void build_ai_cells(struct ai_cell *cells, int y, int x);
 static int ai_cell_compare(struct ai_cell *cell, int dy, int dx);
@@ -247,7 +245,6 @@ static void get_chase_prefs(int mon, int *pref_y, int *pref_x)
 	}
 }
 
-#if 0
 /* get_seeking_prefs()
  *
  * Does all the work of finding the best (or least-bad) square for a seeking
@@ -256,18 +253,114 @@ static void get_chase_prefs(int mon, int *pref_y, int *pref_x)
 
 static void get_seeking_prefs(int y, int x, int dy, int dx, int *pref_y, int *pref_x)
 {
+	struct ai_cell ai_cells[8];
+	int i;
+	int ady, adx;
+	int j;
+	int highest_score = -10000;
+	int tryct;
 	int uroom;
 	int mroom;
+	int troom;
+	*pref_y = y;
+	*pref_x = x;
+	ady = dy > 0 ? dy : -dy;
+	adx = dx > 0 ? dx : -dx;
+	build_ai_cells(ai_cells, y, x);
 	uroom = roomnums[u.y][u.x];
 	mroom = roomnums[y][x];
-	if (uroom == mroom)
+	for (i = 0; i < 8; i++)
 	{
-		/* This should only ever happen if you're both in corridors!
-		 * We shouldn't be *in* this function if you're in the same
-		 * room as he is. */
+		troom = roomnums[ai_cells[i].y][ai_cells[i].x];
+		ai_cells[i].dy = u.y - ai_cells[i].y;
+		ai_cells[i].dx = u.x - ai_cells[i].x;
+		/* Scoring factors:
+		 * If player in a room:
+		 *   Square in player's room: +10
+		 *   Square closer to player: +1
+		 *   Square further from player: -1
+		 *   Square in room with linkage 1 to player's room: +5
+		 * If player in corridor:
+		 *   Square closer to player: +1
+		 *   Square matches one of player's coords: +1
+		 *   Square further from player: -1
+		 * Note that now that wraiths are smart and can walk through
+		 * walls, the player can only finally evade a wraith by
+		 * killing it or leaving the level; fortunately, wraiths 
+		 * are slow. (Danenth will feature ethereal beings, in the
+		 * form of chthonic spirits, which are *not* slow.)
+		 */
+		if (!mon_can_pass(mapmonster[y][x], ai_cells[i].y, ai_cells[i].x))
+		{
+			/* Square impassable to this monster. Set score WAY
+			 * out of bounds and continue. */
+			ai_cells[i].score = -10000;
+			continue;
+		}
+		if (uroom != -1)
+		{
+			/* Player in a room */
+			if (troom == uroom)
+			{
+				ai_cells[i].score += 10;
+			}
+			else if ((troom != -1) && (roomlinkage[troom][uroom] == 1))
+			{
+				ai_cells[i].score += 5;
+			}
+			j = ai_cell_compare(ai_cells + i, dy, dx);
+			if (j > 0)
+			{
+				ai_cells[i].score -= 1;
+			}
+			else if (j < 0)
+			{
+				ai_cells[i].score += 1;
+			}
+			if (ai_cells[i].score > highest_score)
+			{
+				highest_score = ai_cells[i].score;
+			}
+		}
+		else
+		{
+			/* Player in a corridor */
+			if ((ai_cells[i].y == u.y) || (ai_cells[i].x == u.x))
+			{
+				ai_cells[i].score += 1;
+			}
+			j = ai_cell_compare(ai_cells + i, dy, dx);
+			if (j > 0)
+			{
+				ai_cells[i].score -= 1;
+			}
+			else if (j < 0)
+			{
+				ai_cells[i].score += 1;
+			}
+			if (ai_cells[i].score > highest_score)
+			{
+				highest_score = ai_cells[i].score;
+			}
+		}
 	}
+	if (highest_score == -10000)
+	{
+		/* No good targets. */
+		return;
+	}
+	for (tryct = 0; tryct < 32; tryct++)
+	{
+		i = zero_die(8);
+		if (ai_cells[i].score == highest_score)
+		{
+			*pref_y = ai_cells[i].y;
+			*pref_x = ai_cells[i].x;
+			break;
+		}
+	}
+	return;
 }
-#endif
 
 /* get_naive_prefs()
  *
@@ -358,13 +451,6 @@ static void get_naive_prefs(int y, int x, int dy, int dx, int *pref_y, int *pref
 	}
 }
 
-/*
-
-static void get_missile_prefs(int y, int x, int dy, int dx, int *pref_y, int *pref_x)
-{
-}
-*/
-
 /* XXX build_ai_cells()
  *
  * Populate array of eight AI cell descriptors.
@@ -662,35 +748,13 @@ void select_space(int *py, int *px, int dy, int dx, int selection_mode)
 		}
 		break;
 	case 4:
-		/* For now, smart monsters fall through to the non-stupid
-		 * monster case; I lack the patience to implement the
-		 * pathfinding AI right now. */
-#if 0
 		/* "Seeking" monster i.e. monster moving while it can't see
-		 * you, but thinks it knows where you are. */
+		 * you, but thinks it knows where you are. This AI isn't
+		 * great, but it'll do for now. */
 		get_seeking_prefs(*py, *px, dy, dx, ai_y, ai_x);
-		if (mon_can_pass(mapmonster[*py][*px], ai_y[0], ai_x[0]))
-		{
-			y = ai_y[0];
-			x = ai_x[0];
-		}
-		else if (mon_can_pass(mapmonster[*py][*px], ai_y[1], ai_x[1]))
-		{
-			y = ai_y[1];
-			x = ai_x[1];
-		}
-		else if (mon_can_pass(mapmonster[*py][*px], ai_y[2], ai_x[2]))
-		{
-			y = ai_y[2];
-			x = ai_x[2];
-		}
-		else
-		{
-			y = *py;
-			x = *px;
-		}
+		y = ai_y[0];
+		x = ai_x[0];
 		break;
-#endif
 	case 5:
 		/* "chase" AI i.e. pursue your last known position. */
 		get_chase_prefs(mapmonster[*py][*px], ai_y, ai_x);
@@ -832,12 +896,11 @@ void mon_acts(int mon)
 		{
 			return;
 		}
-/*		if (pmon_is_smart(mptr->mon_id))
+		if (pmon_is_smart(mptr->mon_id))
 		{
 			select_space(&y, &x, dy, dx, 4);
 		}
-		else */
-		if (pmon_is_stupid(mptr->mon_id) || (mptr->ai_lasty == -1))
+		else if (pmon_is_stupid(mptr->mon_id) || (mptr->ai_lasty == -1))
 		{
 			select_space(&y, &x, dy, dx, 3);
 		}
