@@ -26,9 +26,11 @@
 
 #define OBJECTS_C
 #include "dunbash.h"
+#include "monsters.h"
 
 struct obj objects[100];
 int get_random_pobj(void);
+static int consume_obj(int obj);
 
 const char ring_colours[20][16] = {
 	"gold", "ruby", "sapphire", "ivory", "coral",
@@ -98,6 +100,16 @@ int read_scroll(int obj)
 			print_msg("You hear a snarl of frustration.\n");
 		}
 		break;
+	case PO_SCR_AGGRAVATE:
+		print_msg("You hear a high-pitched humming noise.\n");
+		for (i = 0; i < 100; i++)
+		{
+			if (monsters[i].used)
+			{
+				monsters[i].awake = 1;
+			}
+		}
+		break;
 	case PO_SCR_PROTECTION:
 		print_msg("You feel like something is helping you.\n");
 		if (!u.protection)
@@ -127,21 +139,42 @@ int read_scroll(int obj)
 		return 0;
 	}
 	permobjs[optr->obj_id].known = 1;
-	optr->quan--;
-	if (optr->quan < 1)
-	{
-		optr->used = 0;
-		return 1;
-	}
-	return 0;
+	return consume_obj(obj);
 }
 
 static int consume_obj(int obj)
 {
+	int i;
 	objects[obj].quan--;
 	if (objects[obj].quan == 0)
 	{
 		objects[obj].used = 0;
+		if (objects[obj].with_you)
+		{
+			if (obj == u.armour)
+			{
+				u.armour = -1;
+				recalc_defence();
+			}
+			else if (obj == u.weapon)
+			{
+				u.weapon = -1;
+				recalc_defence();
+			}
+			else if (obj == u.ring)
+			{
+				u.ring = -1;
+				recalc_defence();
+			}
+			for (i = 0; i < 19; i++)
+			{
+				if (u.inventory[i] == obj)
+				{
+					u.inventory[i] = -1;
+					break;
+				}
+			}
+		}
 		return 1;
 	}
 	return 0;
@@ -154,7 +187,17 @@ int eat_food(int obj)
 	{
 		return -1;
 	}
+	if (u.food < 0)
+	{
+		print_msg("You ravenously devour the food ration!\n");
+	}
+	else
+	{
+		print_msg("You eat some food.\n");
+	}
 	u.food += 2000;
+	status_updated = 1;
+	display_update();
 	return consume_obj(obj);
 }
 
@@ -176,8 +219,8 @@ int quaff_potion(int obj)
 		break;
 	case PO_POT_WEAKNESS:
 		print_msg("You feel that was a bad idea!\n");
-		drain_body(one_die(4), "a potion of weakness");
-		drain_agility(one_die(4), "a potion of weakness");
+		drain_body(one_die(4), "a potion of weakness", 1);
+		drain_agility(one_die(4), "a potion of weakness", 1);
 		break;
 	case PO_POT_POISON:
 		print_msg("This is poison!\n");
@@ -189,18 +232,41 @@ int quaff_potion(int obj)
 		 * hit point. */
 		heal_u(dice(3, 12), 1);
 		break;
+	case PO_POT_RESTORATION:
+		print_msg("This potion makes you feel warm all over.\n");
+		if (!zero_die(2))
+		{
+			if (u.adam)
+			{
+				print_msg("You feel less clumsy.\n");
+				u.adam = 0;
+			}
+			else if (u.bdam)
+			{
+				print_msg("You feel less feeble.\n");
+				u.bdam = 0;
+			}
+		}
+		else
+		{
+			if (u.bdam)
+			{
+				print_msg("You feel less feeble.\n");
+				u.bdam = 0;
+			}
+			else if (u.adam)
+			{
+				print_msg("You feel less clumsy.\n");
+				u.adam = 0;
+			}
+		}
+		break;
 	default:
 		print_msg("Impossible: quaffing non-potion\n");
 		return 0;
 	}
 	permobjs[optr->obj_id].known = 1;
-	optr->quan--;
-	if (optr->quan < 1)
-	{
-		optr->used = 0;
-		return 1;
-	}
-	return 0;
+	return consume_obj(obj);
 }
 
 void flavours_init(void)
@@ -256,6 +322,8 @@ void flavours_init(void)
 	permobjs[PO_SCR_TELEPORT].power = colour_choices[1];
 	permobjs[PO_SCR_MONSTERS].power = colour_choices[2];
 	permobjs[PO_SCR_IDENTIFY].power = colour_choices[3];
+	permobjs[PO_SCR_PROTECTION].power = colour_choices[4];
+	permobjs[PO_SCR_AGGRAVATE].power = colour_choices[5];
 	/* Potions */
 	for (i = 0; i < 10;)
 	{
@@ -278,6 +346,7 @@ void flavours_init(void)
 	permobjs[PO_POT_POISON].power = colour_choices[2];
 	permobjs[PO_POT_AGILITY].power = colour_choices[3];
 	permobjs[PO_POT_WEAKNESS].power = colour_choices[4];
+	permobjs[PO_POT_RESTORATION].power = colour_choices[5];
 }
 
 int create_obj_class(enum poclass_num po_class, int quantity, int with_you, int y, int x)
@@ -386,6 +455,18 @@ int create_obj(int po_idx, int quantity, int with_you, int y, int x)
 	{
 		objects[i].quan = quantity;
 	}
+	switch (permobjs[po_idx].poclass)
+	{
+	case POCLASS_WEAPON:
+	case POCLASS_ARMOUR:
+		/* Set durability of weapons and armour to a suitable value.
+		 * 100 has been chosen for the moment, but this may need
+		 * tuning. */
+		objects[i].durability = OBJ_MAX_DUR;
+		break;
+	default:
+		break;
+	}
 	if (!objects[i].with_you)
 	{
 		mapobject[y][x] = i;
@@ -393,6 +474,84 @@ int create_obj(int po_idx, int quantity, int with_you, int y, int x)
 	return i;
 }
 
+void fprint_obj_name(FILE *fp, int obj)
+{
+	struct obj *optr;
+	struct permobj *poptr;
+	optr = objects + obj;
+	poptr = permobjs + optr->obj_id;
+	if (poptr->known)
+	{
+		if (optr->quan > 1)
+		{
+			fprintf(fp, "%d %s", optr->quan, poptr->plural);
+		}
+		else if (po_is_stackable(optr->obj_id))
+		{
+			fprintf(fp, "1 %s", poptr->name);
+		}
+		else if ((poptr->poclass == POCLASS_WEAPON) ||
+			 (poptr->poclass == POCLASS_ARMOUR))
+		{
+			fprintf(fp, "a%s %s (%d/%d)", is_vowel(poptr->name[0]) ? "n" : "", poptr->name, optr->durability, OBJ_MAX_DUR);
+		}
+		else
+		{
+			fprintf(fp, "a%s %s", is_vowel(poptr->name[0]) ? "n" : "", poptr->name);
+		}
+	}
+	else
+	{
+		switch (poptr->poclass)
+		{
+		case POCLASS_NONE:
+			fprintf(fp, "a non-thing (%d)", optr->obj_id);
+			break;
+		case POCLASS_FOOD:
+			fprintf(fp, "a mysterious food (%d)", optr->obj_id);
+			break;
+		case POCLASS_WEAPON:
+			fprintf(fp, "a mysterious weapon (%d)", optr->obj_id);
+			break;
+		case POCLASS_ARMOUR:
+			if ((optr->obj_id == PO_ROBE) ||
+			    (optr->obj_id == PO_ROBE_SHADOWS) ||
+			    (optr->obj_id == PO_ROBE_SWIFTNESS))
+			{
+				fprintf(fp, "a robe");
+			}
+			else
+			{
+				fprintf(fp, "some mysterious armour (%d)", optr->obj_id);
+			}
+			break;
+		case POCLASS_SCROLL:
+			if (optr->quan > 1)
+			{
+				fprintf(fp, "%d scrolls '%s'", optr->quan, scroll_titles[poptr->power]);
+			}
+			else
+			{
+				fprintf(fp, "1 scroll '%s'", scroll_titles[poptr->power]);
+			}
+			break;
+		case POCLASS_POTION:
+			if (optr->quan > 1)
+			{
+				fprintf(fp, "%d %s potions", optr->quan, potion_colours[poptr->power]);
+			}
+			else
+			{
+				fprintf(fp, "1 %s potion", potion_colours[poptr->power]);
+			}
+			break;
+		case POCLASS_RING:
+			fprintf(fp, "a%s %s ring", is_vowel(ring_colours[poptr->power][0]) ? "n" : "", ring_colours[poptr->power]);
+			break;
+		}
+	}
+}
+
 void print_obj_name(int obj)
 {
 	struct obj *optr;
@@ -409,6 +568,11 @@ void print_obj_name(int obj)
 		{
 			print_msg("1 %s", poptr->name);
 		}
+		else if ((poptr->poclass == POCLASS_WEAPON) ||
+			 (poptr->poclass == POCLASS_ARMOUR))
+		{
+			print_msg("a%s %s (%d/%d)", is_vowel(poptr->name[0]) ? "n" : "", poptr->name, optr->durability, OBJ_MAX_DUR);
+		}
 		else
 		{
 			print_msg("a%s %s", is_vowel(poptr->name[0]) ? "n" : "", poptr->name);
@@ -561,4 +725,27 @@ void attempt_pickup(void)
 	print_msg("\n");
 }
 
+void damage_obj(int obj)
+{
+	/* Only weapons and armour have non-zero durability. */
+	if (objects[obj].durability == 0)
+	{
+		/* Break the object. Weapons and armour don't stack. */
+		if (obj == u.weapon)
+		{
+			print_msg("Your weapon breaks!\n");
+		}
+		else if (obj == u.armour)
+		{
+			print_msg("Your armour is ruined!\n");
+		}
+		consume_obj(obj);
+		recalc_defence();
+	}
+	else
+	{
+		objects[obj].durability--;
+	}
+} 
+
 /* objects.c */
