@@ -106,7 +106,7 @@ struct permon permons[NUM_OF_PERMONS] = {
 	[PM_ARCHMAGE] =
 	{
 		/* Uses black magic against you; see bmagic.c for details. */
-		"archmage", "archmagi", 'A', DBCLR_L_BLUE, 80, 24, 80, 15, 30, 15, 15, DT_ELEC, "casts", 15, 1500, 1
+		"archmage", "archmagi", 'w', DBCLR_L_BLUE, 80, 24, 80, 15, 30, 15, 15, DT_ELEC, "casts", 15, 1500, 1
 	},
 	[PM_ZOMBIE] =
 	{
@@ -149,12 +149,12 @@ struct permon permons[NUM_OF_PERMONS] = {
 	},
 	[PM_ICE_MONSTER] =
 	{
-		/* Fires ice blasts (ring of frost resists) */
+		/* Fires ice blasts. */
 		"ice monster", "ice monsters", 'I', DBCLR_WHITE, 50, 6, 40, 10, 20, 15, 15, DT_COLD, "launches a blast of", 15, 35, 0
 	},
 	[PM_DRAGON] =
 	{
-		/* Breathes fire (ring of fire resists). */
+		/* Breathes fire. */
 		"dragon", "dragons", 'D', DBCLR_RED, 50, 15, 80, 20, 20, 20, 20, DT_FIRE, "breathes", 20, 300, 1
 	},
 };
