@@ -42,7 +42,7 @@ void recalc_defence(void)
 	u.speed = (u.leadfoot ? 0 : 1);
 	if (u.armour != -1)
 	{
-		u.defence = permobjs[objects[u.armour].obj_id].power;
+		u.defence = u.armourmelt ? 0 : permobjs[objects[u.armour].obj_id].power;
 		u.defence += u.withering ? (u.agility / 10) : (u.agility / 5);
 		switch (objects[u.armour].obj_id)
 		{
@@ -157,11 +157,18 @@ int reloc_player(int y, int x)
 	return 0;
 }
 
-int drain_body(int amount, const char *what)
+int drain_body(int amount, const char *what, int permanent)
 {
 	print_msg("You feel weaker!\n");
-	u.body -= amount;
-	if (u.body < 0)
+	if (permanent)
+	{
+		u.body -= amount;
+	}
+	else
+	{
+		u.bdam += amount;
+	}
+	if ((u.body - u.bdam) < 0)
 	{
 		print_msg("Your heart is too weak to beat.\n");
 		do_death(DEATH_BODY, what);
@@ -170,11 +177,18 @@ int drain_body(int amount, const char *what)
 	return 0;
 }
 
-int drain_agility(int amount, const char *what)
+int drain_agility(int amount, const char *what, int permanent)
 {
 	print_msg("You feel clumsy!\n");
-	u.agility -= amount;
-	if (u.agility < 0)
+	if (permanent)
+	{
+		u.agility -= amount;
+	}
+	else
+	{
+		u.adam += amount;
+	}
+	if ((u.agility - u.adam) < 0)
 	{
 		print_msg("You forget how to breathe.\n");
 		do_death(DEATH_AGILITY, what);
@@ -248,13 +262,70 @@ void do_death(enum death d, const char *what)
 
 }
 
+void write_char_dump(void)
+{
+	FILE *fp;
+	char filename[32];
+	int i;
+	snprintf(filename, 31, "%s.dump", u.name);
+	fp = fopen(filename, "w");
+	if (fp == NULL)
+	{
+		print_msg("Couldn't create dump file. Dump failed.\n");
+		return;
+	}
+	fprintf(fp, "%s, level %d adventurer (%d XP)\n", u.name, u.level, u.experience);
+	fprintf(fp, "%d gold pieces collected.\n", u.gold);
+	fprintf(fp, "%d of %d hit points.\n", u.hpcur, u.hpmax);
+	fprintf(fp, "Body %d (%d damage).\n", u.body, u.bdam);
+	fprintf(fp, "Agility %d (%d damage).\n", u.agility, u.adam);
+	fprintf(fp, "Defence %d.\n", u.defence);
+	fprintf(fp, "Inventory:\n");
+	for (i = 0; i < 19; i++)
+	{
+		if (u.inventory[i] != -1)
+		{
+			fprint_obj_name(fp, u.inventory[i]);
+			fputc('\n', fp);
+		}
+	}
+	fflush(fp);
+	fclose(fp);
+}
+
 void u_init(void)
 {
+	char * hasslash = NULL;
 	u.name[16] = '\0';
-	print_msg("What is your name, stranger?\n");
-	read_input(u.name, 16);
+	do {
+		print_msg("What is your name, stranger?\n");
+		read_input(u.name, 16);
+		hasslash = strchr(u.name, '/');
+		/* Now that we create a named dump file, we must not
+		 * permit the player's name to contain a slash, colon,
+		 * or backslash. */
+		if (hasslash)
+		{
+			print_msg("No slashes permitted.\n");
+			continue;
+		}
+		hasslash = strchr(u.name, '\\');
+		if (hasslash)
+		{
+			print_msg("No backslashes permitted.\n");
+			continue;
+		}
+		hasslash = strchr(u.name, ':');
+		if (hasslash)
+		{
+			print_msg("No colons permitted.\n");
+			continue;
+		}
+	} while (hasslash != NULL);
 	u.body = 10;
+	u.bdam = 0;
 	u.agility = 10;
+	u.adam = 0;
 	u.hpmax = 20;
 	u.hpcur = 20;
 	u.experience = 0;
@@ -285,7 +356,7 @@ int lev_threshold(int level)
 	}
 	if (level < 30)
 	{
-		return 100000 * (level - 19);
+		return 100000 * (level - 18);
 	}
 	return INT_MAX;
 }
